@@ -24,6 +24,7 @@
 #include "gappinfo.h"
 #include "glibintl.h"
 #include <gioerror.h>
+#include <gfile.h>
 
 #include "gioalias.h"
 
@@ -136,9 +137,12 @@ g_app_info_equal (GAppInfo *appinfo1,
  * 
  * Gets the ID of an application. An id is a string that
  * identifies the application. The exact format of the id is
- * platform dependent. For instance on Unix this is the
+ * platform dependent. For instance, on Unix this is the
  * desktop file id from the xdg menu specification.
  *
+ * Note that the returned ID may be %NULL, depending on how
+ * the @appinfo has been constructed.
+ *
  * Returns: a string containing the application's ID.
  **/
 const char *
@@ -180,8 +184,7 @@ g_app_info_get_name (GAppInfo *appinfo)
  * Gets a human-readable description of an installed application.
  *
  * Returns: a string containing a description of the 
- * application @appinfo, or %NULL if none. The returned string should be not freed 
- * when no longer needed.
+ * application @appinfo, or %NULL if none. 
  **/
 const char *
 g_app_info_get_description (GAppInfo *appinfo)
@@ -395,7 +398,7 @@ g_app_info_get_icon (GAppInfo *appinfo)
  * 
  * Launches the application. Passes @files to the launched application 
  * as arguments, using the optional @launch_context to get information
- * about the details of the launcher (like what screen its is on).
+ * about the details of the launcher (like what screen it is on).
  * On error, @error will be set accordingly.
  *
  * To lauch the application without arguments pass a %NULL @files list.
@@ -404,6 +407,11 @@ g_app_info_get_icon (GAppInfo *appinfo)
  * can fail to start if it runs into problems during startup. There is
  * no way to detect this.
  *
+ * Some URIs can be changed when passed through a GFile (for instance
+ * unsupported uris with strange formats like mailto:), so if you have
+ * a textual uri you want to pass in as argument, consider using
+ * g_app_info_launch_uris() instead.
+ * 
  * Returns: %TRUE on successful launch, %FALSE otherwise. 
  **/
 gboolean
@@ -443,6 +451,27 @@ g_app_info_supports_uris (GAppInfo *appinfo)
 }
 
 
+/**
+ * g_app_info_supports_files:
+ * @appinfo: a #GAppInfo.
+ * 
+ * Checks if the application accepts files as arguments.
+ *
+ * Returns: %TRUE if the @appinfo supports files.
+ **/
+gboolean
+g_app_info_supports_files (GAppInfo *appinfo)
+{
+  GAppInfoIface *iface;
+  
+  g_return_val_if_fail (G_IS_APP_INFO (appinfo), FALSE);
+
+  iface = G_APP_INFO_GET_IFACE (appinfo);
+
+  return (* iface->supports_files) (appinfo);
+}
+
+
 /**
  * g_app_info_launch_uris:
  * @appinfo: a #GAppInfo.
@@ -452,7 +481,7 @@ g_app_info_supports_uris (GAppInfo *appinfo)
  * 
  * Launches the application. Passes @uris to the launched application 
  * as arguments, using the optional @launch_context to get information
- * about the details of the launcher (like what screen its is on).
+ * about the details of the launcher (like what screen it is on).
  * On error, @error will be set accordingly.
  *
  * To lauch the application without arguments pass a %NULL @uris list.
@@ -475,27 +504,21 @@ g_app_info_launch_uris (GAppInfo           *appinfo,
 
   iface = G_APP_INFO_GET_IFACE (appinfo);
 
-  return (* iface->launch) (appinfo, uris, launch_context, error);
+  return (* iface->launch_uris) (appinfo, uris, launch_context, error);
 }
 
 
 /**
  * g_app_info_should_show:
  * @appinfo: a #GAppInfo.
- * @desktop_env: A string specifying what desktop this is, or %NULL.
  *
- * Checks if the application info should be shown when listing
- * applications available.
- *
- * @destkop_env is used to hide applications that are specified to
- * just show up in specific desktops. For instance, passing in "GNOME"
- * would show all applications specific to the Gnome desktop.
+ * Checks if the application info should be shown in menus that 
+ * list available applications.
  * 
  * Returns: %TRUE if the @appinfo should be shown, %FALSE otherwise.
  **/
 gboolean
-g_app_info_should_show (GAppInfo   *appinfo,
-			const char *desktop_env)
+g_app_info_should_show (GAppInfo *appinfo)
 {
   GAppInfoIface *iface;
   
@@ -503,16 +526,60 @@ g_app_info_should_show (GAppInfo   *appinfo,
 
   iface = G_APP_INFO_GET_IFACE (appinfo);
 
-  return (* iface->should_show) (appinfo, desktop_env);
+  return (* iface->should_show) (appinfo);
 }
 
+/**
+ * g_app_info_launch_default_for_uri:
+ * @uri: the uri to show
+ * @launch_context: an optional #GAppLaunchContext.
+ * @error: a #GError.
+ *
+ * Utility function that launches the default application 
+ * registered to handle the specified uri. Synchronous I/O
+ * is done on the uri to detext the type of the file if
+ * required.
+ * 
+ * Returns: %TRUE on success, %FALSE on error.
+ **/
+gboolean
+g_app_info_launch_default_for_uri (const char         *uri,
+				   GAppLaunchContext  *launch_context,
+				   GError            **error)
+{
+  GAppInfo *app_info;
+  GFile *file;
+  GList l;
+  gboolean res;
+
+  file = g_file_new_for_uri (uri);
+  app_info = g_file_query_default_handler (file, NULL, error);
+  g_object_unref (file);
+  if (app_info == NULL)
+    return FALSE;
+
+  /* Use the uri, not the GFile, as the GFile roundtrip may
+   * affect the uri which we don't want (for instance for a
+   * mailto: uri).
+   */
+  l.data = (char *)uri;
+  l.next = l.prev = NULL;
+  res = g_app_info_launch_uris (app_info, &l,
+				launch_context, error);
+
+  g_object_unref (app_info);
+  
+  return res;
+}
+
+
 G_DEFINE_TYPE (GAppLaunchContext, g_app_launch_context, G_TYPE_OBJECT);
 
 /**
  * g_app_launch_context_new:
  * 
  * Creates a new application launch context. This is not normally used,
- * instead you instantiate a subclass of this, such as GdkLaunchContext.
+ * instead you instantiate a subclass of this, such as #GdkAppLaunchContext.
  *
  * Returns: a #GAppLaunchContext.
  **/
