@@ -33,6 +33,7 @@
 #include <gio/gfileinputstream.h>
 #include <gio/gfileoutputstream.h>
 #include <gio/gmountoperation.h>
+#include <gio/gappinfo.h>
 
 G_BEGIN_DECLS
 
@@ -66,13 +67,25 @@ typedef enum  {
   G_FILE_CREATE_PRIVATE = (1<<0)
 } GFileCreateFlags;
 
+
+/**
+ * GMountMountFlags:
+ * @G_MOUNT_MOUNT_NONE: No flags set.
+ * 
+ * Flags used when mounting a mount.
+ */
+typedef enum  {
+  G_MOUNT_MOUNT_NONE = 0,
+} GMountMountFlags;
+
+
 /**
  * GMountUnmountFlags:
  * @G_MOUNT_UNMOUNT_NONE: No flags set.
  * @G_MOUNT_UNMOUNT_FORCE: Unmount even if there are outstanding
  *  file operations on the mount.
  * 
- * Flags used when an operation may create a file.
+ * Flags used when an unmounting a mount.
  */
 typedef enum  {
   G_MOUNT_UNMOUNT_NONE = 0,
@@ -492,6 +505,7 @@ struct _GFileIface
 
 
   void                (*mount_mountable)           (GFile               *file,
+						    GMountMountFlags     flags,
 						    GMountOperation     *mount_operation,
 						    GCancellable         *cancellable,
 						    GAsyncReadyCallback  callback,
@@ -518,6 +532,7 @@ struct _GFileIface
 
 
   void     (*mount_enclosing_volume)        (GFile *location,
+					     GMountMountFlags flags,
 					     GMountOperation *mount_operation,
 					     GCancellable *cancellable,
 					     GAsyncReadyCallback callback,
@@ -787,6 +802,7 @@ gboolean                g_file_set_attribute_int64        (GFile
 							   GCancellable               *cancellable,
 							   GError                    **error);
 void                    g_file_mount_enclosing_volume     (GFile                      *location,
+							   GMountMountFlags            flags,
 							   GMountOperation            *mount_operation,
 							   GCancellable               *cancellable,
 							   GAsyncReadyCallback         callback,
@@ -795,6 +811,7 @@ gboolean                g_file_mount_enclosing_volume_finish (GFile
 							   GAsyncResult               *result,
 							   GError                    **error);
 void                    g_file_mount_mountable            (GFile                      *file,
+							   GMountMountFlags            flags,
 							   GMountOperation            *mount_operation,
 							   GCancellable               *cancellable,
 							   GAsyncReadyCallback         callback,
@@ -838,6 +855,9 @@ GFileMonitor*           g_file_monitor_file               (GFile
 
 /* Utilities */
 
+GAppInfo *g_file_query_default_handler       (GFile                  *file,
+					      GCancellable           *cancellable,
+					      GError                **error);
 gboolean g_file_load_contents                (GFile                  *file,
 					      GCancellable           *cancellable,
 					      char                  **contents,
