@@ -94,7 +94,7 @@
 
 #include "gioalias.h"
 
-static void g_local_file_file_iface_init (GFileIface       *iface);
+static void g_local_file_file_iface_init (GFileIface *iface);
 
 static GFileAttributeInfoList *local_writable_attributes = NULL;
 static GFileAttributeInfoList *local_writable_namespaces = NULL;
@@ -111,7 +111,7 @@ G_DEFINE_TYPE_WITH_CODE (GLocalFile, g_local_file, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (G_TYPE_FILE,
 						g_local_file_file_iface_init))
 
-static char * find_mountpoint_for (const char *file, dev_t dev);
+static char *find_mountpoint_for (const char *file, dev_t dev);
 
 static void
 g_local_file_finalize (GObject *object)
@@ -214,6 +214,7 @@ canonicalize_filename (const char *filename)
 {
   char *canon, *start, *p, *q;
   char *cwd;
+  int i;
   
   if (!g_path_is_absolute (filename))
     {
@@ -226,6 +227,24 @@ canonicalize_filename (const char *filename)
 
   start = (char *)g_path_skip_root (canon);
 
+  /* POSIX allows double slashes at the start to
+   * mean something special (as does windows too).
+   * So, "//" != "/", but more than two slashes
+   * is treated as "/".
+   */
+  i = 0;
+  for (p = start - 1;
+       (p >= canon) &&
+	 G_IS_DIR_SEPARATOR (*p);
+       p--)
+    i++;
+  if (i > 2)
+    {
+      i -= 1;
+      start -= i;
+      memmove (start, start+i, strlen (start+i)+1);
+    }
+  
   p = start;
   while (*p != 0)
     {
@@ -348,14 +367,14 @@ name_is_valid_for_display (const char *string,
 			   gboolean    is_valid_utf8)
 {
   char c;
-  
+
   if (!is_valid_utf8 &&
       !g_utf8_validate (string, -1, NULL))
     return FALSE;
 
   while ((c = *string++) != 0)
     {
-      if (g_ascii_iscntrl(c))
+      if (g_ascii_iscntrl (c))
 	return FALSE;
     }
 
@@ -372,7 +391,8 @@ g_local_file_get_parse_name (GFile *file)
   char *roundtripped_filename;
   gboolean free_utf8_filename;
   gboolean is_valid_utf8;
-
+  char *escaped_path;
+  
   filename = G_LOCAL_FILE (file)->filename;
   if (get_filename_charset (&charset))
     {
@@ -402,7 +422,6 @@ g_local_file_get_parse_name (GFile *file)
 	}
     }
 
-
   if (utf8_filename != NULL &&
       name_is_valid_for_display (utf8_filename, is_valid_utf8))
     {
@@ -413,7 +432,16 @@ g_local_file_get_parse_name (GFile *file)
     }
   else
     {
-      parse_name = g_filename_to_uri (filename, NULL, NULL);
+      escaped_path = g_uri_escape_string (filename,
+					  G_URI_RESERVED_CHARS_ALLOWED_IN_PATH_ELEMENT "/",
+					  TRUE);
+      parse_name = g_strconcat ("file://",
+				(*escaped_path != '/') ? "/" : "",
+				escaped_path,
+				NULL);
+      
+      g_free (escaped_path);
+
       if (free_utf8_filename)
 	g_free (utf8_filename);
     }
@@ -475,6 +503,13 @@ match_prefix (const char *path,
   prefix_len = strlen (prefix);
   if (strncmp (path, prefix, prefix_len) != 0)
     return NULL;
+  
+  /* Handle the case where prefix is the root, so that
+   * the IS_DIR_SEPRARATOR check below works */
+  if (prefix_len > 0 &&
+      G_IS_DIR_SEPARATOR (prefix[prefix_len-1]))
+    prefix_len--;
+  
   return path + prefix_len;
 }
 
@@ -677,7 +712,7 @@ static gboolean
 device_equal (gconstpointer v1,
               gconstpointer v2)
 {
-  return *(dev_t *)v1 == * (dev_t *)v2;
+  return *(dev_t *)v1 == *(dev_t *)v2;
 }
 
 static guint
@@ -805,10 +840,12 @@ g_local_file_query_filesystem_info (GFile         *file,
 
   if (statfs_result == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error getting filesystem info: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return NULL;
     }
 
@@ -929,7 +966,7 @@ g_local_file_set_display_name (GFile         *file,
       return NULL;
     }
   
-  new_file = g_file_get_child_for_display_name  (parent, display_name, error);
+  new_file = g_file_get_child_for_display_name (parent, display_name, error);
   g_object_unref (parent);
   
   if (new_file == NULL)
@@ -1018,13 +1055,13 @@ g_local_file_query_writable_namespaces (GFile         *file,
 }
 
 static gboolean
-g_local_file_set_attribute (GFile                      *file,
-			    const char                 *attribute,
-			    GFileAttributeType          type,
-			    gpointer                    value_p,
-			    GFileQueryInfoFlags         flags,
-			    GCancellable               *cancellable,
-			    GError                    **error)
+g_local_file_set_attribute (GFile                *file,
+			    const char           *attribute,
+			    GFileAttributeType    type,
+			    gpointer              value_p,
+			    GFileQueryInfoFlags   flags,
+			    GCancellable         *cancellable,
+			    GError              **error)
 {
   GLocalFile *local = G_LOCAL_FILE (file);
 
@@ -1046,7 +1083,7 @@ g_local_file_set_attributes_from_info (GFile                *file,
 {
   GLocalFile *local = G_LOCAL_FILE (file);
   int res, chained_res;
-  GFileIface* default_iface;
+  GFileIface *default_iface;
 
   res = _g_local_file_info_set_attributes (local->filename,
 					   info, flags, 
@@ -1075,10 +1112,12 @@ g_local_file_read (GFile         *file,
   fd = g_open (local->filename, O_RDONLY, 0);
   if (fd == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error opening file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return NULL;
     }
 
@@ -1137,10 +1176,12 @@ g_local_file_delete (GFile         *file,
   
   if (g_remove (local->filename) == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error removing file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
@@ -1325,7 +1366,7 @@ get_unique_filename (const char *basename,
 
   dot = strchr (basename, '.');
   if (dot)
-    return g_strdup_printf ("%.*s.%d%s", dot - basename, basename, id, dot);
+    return g_strdup_printf ("%.*s.%d%s", (int)(dot - basename), basename, id, dot);
   else
     return g_strdup_printf ("%s.%d", basename, id);
 }
@@ -1429,10 +1470,12 @@ g_local_file_trash (GFile         *file,
   
   if (g_lstat (local->filename, &file_stat) != 0)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error trashing file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
     
@@ -1449,14 +1492,13 @@ g_local_file_trash (GFile         *file,
       if (g_mkdir_with_parents (trashdir, 0700) < 0)
 	{
           char *display_name;
-          int err;
+          int errsv = errno;
 
-          err = errno;
           display_name = g_filename_display_name (trashdir);
           g_set_error (error, G_IO_ERROR,
-                       g_io_error_from_errno (err),
+                       g_io_error_from_errno (errsv),
                        _("Unable to create trash dir %s: %s"),
-                       display_name, g_strerror (err));
+                       display_name, g_strerror (errsv));
           g_free (display_name);
           g_free (trashdir);
           return FALSE;
@@ -1511,25 +1553,45 @@ g_local_file_trash (GFile         *file,
 
       if (trashdir == NULL)
 	{
+	  gboolean tried_create;
+	  
 	  /* No global trash dir, or it failed the tests, fall back to $topdir/.Trash-$uid */
 	  dirname = g_strdup_printf (".Trash-%s", uid_str);
 	  trashdir = g_build_filename (topdir, dirname, NULL);
 	  g_free (dirname);
-	  
+
+	  tried_create = FALSE;
+
+	retry:
 	  if (g_lstat (trashdir, &trash_stat) == 0)
 	    {
 	      if (!S_ISDIR (trash_stat.st_mode) ||
 		  trash_stat.st_uid != uid)
 		{
+		  /* Remove the failed directory */
+		  if (tried_create)
+		    g_remove (trashdir);
+		  
 		  /* Not a directory or not owned by user, ignore */
 		  g_free (trashdir);
 		  trashdir = NULL;
 		}
 	    }
-	  else if (g_mkdir (trashdir, 0700) == -1)
+	  else
 	    {
-	      g_free (trashdir);
-	      trashdir = NULL;
+	      if (!tried_create &&
+		  g_mkdir (trashdir, 0700) != -1)
+		{
+		  /* Ensure that the created dir has the right uid etc.
+		     This might fail on e.g. a FAT dir */
+		  tried_create = TRUE;
+		  goto retry;
+		}
+	      else
+		{
+		  g_free (trashdir);
+		  trashdir = NULL;
+		}
 	    }
 	}
 #endif
@@ -1584,15 +1646,17 @@ g_local_file_trash (GFile         *file,
 
   if (fd == -1)
     {
+      int errsv = errno;
+
       g_free (filesdir);
       g_free (topdir);
       g_free (trashname);
       g_free (infofile);
       
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
-		   _("Unable to create trashed file: %s"),
-		   g_strerror (errno));
+		   g_io_error_from_errno (errsv),
+		   _("Unable to create trashing info file: %s"),
+		   g_strerror (errsv));
       return FALSE;
     }
 
@@ -1607,15 +1671,17 @@ g_local_file_trash (GFile         *file,
 
   if (g_rename (local->filename, trashfile) == -1)
     {
+      int errsv = errno;
+
       g_free (topdir);
       g_free (trashname);
       g_free (infofile);
       g_free (trashfile);
       
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Unable to trash file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
 
@@ -1746,24 +1812,39 @@ g_local_file_move (GFile                  *source,
 		   gpointer                progress_callback_data,
 		   GError                **error)
 {
-  GLocalFile *local_source = G_LOCAL_FILE (source);
-  GLocalFile *local_destination = G_LOCAL_FILE (destination);
+  GLocalFile *local_source, *local_destination;
   struct stat statbuf;
   gboolean destination_exist, source_is_dir;
   char *backup_name;
   int res;
-
+  off_t source_size;
+  
+  if (!G_IS_LOCAL_FILE (source) ||
+      !G_IS_LOCAL_FILE (destination))
+    {
+      /* Fall back to default move */
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "Move not supported");
+      return FALSE;
+    }
+  
+  local_source = G_LOCAL_FILE (source);
+  local_destination = G_LOCAL_FILE (destination);
+  
   res = g_lstat (local_source->filename, &statbuf);
   if (res == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error moving file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   else
     source_is_dir = S_ISDIR (statbuf.st_mode);
+
+  source_size = statbuf.st_size;
   
   destination_exist = FALSE;
   res = g_lstat (local_destination->filename, &statbuf);
@@ -1788,7 +1869,7 @@ g_local_file_move (GFile                  *source,
 	  g_set_error (error,
 		       G_IO_ERROR,
 		       G_IO_ERROR_EXISTS,
-		       _("Target file already exists"));
+		       _("Target file exists"));
 	  return FALSE;
 	}
     }
@@ -1816,10 +1897,12 @@ g_local_file_move (GFile                  *source,
       res = unlink (local_destination->filename);
       if (res == -1)
 	{
+          int errsv = errno;
+
 	  g_set_error (error, G_IO_ERROR,
-		       g_io_error_from_errno (errno),
+		       g_io_error_from_errno (errsv),
 		       _("Error removing target file: %s"),
-		       g_strerror (errno));
+		       g_strerror (errsv));
 	  return FALSE;
 	}
     }
@@ -1846,27 +1929,33 @@ g_local_file_move (GFile                  *source,
 		     _("Error moving file: %s"),
 		     g_strerror (errsv));
       return FALSE;
-
     }
+
+  /* Make sure we send full copied size */
+  if (progress_callback)
+    progress_callback (source_size, source_size, progress_callback_data);
+  
   return TRUE;
 }
 
-static GDirectoryMonitor*
+static GFileMonitor*
 g_local_file_monitor_dir (GFile             *file,
 			  GFileMonitorFlags  flags,
-			  GCancellable      *cancellable)
+			  GCancellable      *cancellable,
+			  GError           **error)
 {
   GLocalFile* local_file = G_LOCAL_FILE(file);
-  return _g_local_directory_monitor_new (local_file->filename, flags);
+  return _g_local_directory_monitor_new (local_file->filename, flags, error);
 }
 
 static GFileMonitor*
 g_local_file_monitor_file (GFile             *file,
 			   GFileMonitorFlags  flags,
-			   GCancellable      *cancellable)
+			   GCancellable      *cancellable,
+			   GError           **error)
 {
   GLocalFile* local_file = G_LOCAL_FILE(file);
-  return _g_local_file_monitor_new (local_file->filename, flags);
+  return _g_local_file_monitor_new (local_file->filename, flags, error);
 }
 
 static void
