@@ -546,7 +546,7 @@ get_one_xattr_from_fd (int         fd,
   char *value_p;
   ssize_t len;
 
-  len = g_fgetxattr (fd, xattr, value, sizeof (value)-1);
+  len = g_fgetxattr (fd, xattr, value, sizeof (value) - 1);
 
   value_p = NULL;
   if (len >= 0)
@@ -558,7 +558,7 @@ get_one_xattr_from_fd (int         fd,
       if (len < 0)
 	return;
 
-      value_p = g_malloc (len+1);
+      value_p = g_malloc (len + 1);
 
       len = g_fgetxattr (fd, xattr, value_p, len);
 
@@ -759,7 +759,7 @@ set_xattr (char                       *filename,
       g_set_error (error, G_IO_ERROR,
 		   g_io_error_from_errno (errsv),
 		   _("Error setting extended attribute '%s': %s"),
-		   escaped_attribute, g_strerror (errno));
+		   escaped_attribute, g_strerror (errsv));
       return FALSE;
     }
   
@@ -1072,11 +1072,11 @@ lookup_uid_data (uid_t uid)
       if (data->user_name != NULL)
 	data->real_name = g_strdup (data->user_name);
       else
-	data->real_name = g_strdup_printf("user #%d", (int)uid);
+	data->real_name = g_strdup_printf ("user #%d", (int)uid);
     }
   
   if (data->user_name == NULL)
-    data->user_name = g_strdup_printf("%d", (int)uid);
+    data->user_name = g_strdup_printf ("%d", (int)uid);
   
   g_hash_table_replace (uid_cache, GINT_TO_POINTER (uid), data);
   
@@ -1240,11 +1240,13 @@ get_thumbnail_attributes (const char *path,
 
   checksum = g_checksum_new (G_CHECKSUM_MD5);
   g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+  
+  g_free (uri);
 
   basename = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
   g_checksum_free (checksum);
 
-  filename = g_build_filename (g_get_home_dir(),
+  filename = g_build_filename (g_get_home_dir (),
                                ".thumbnails", "normal", basename,
                                NULL);
 
@@ -1253,7 +1255,7 @@ get_thumbnail_attributes (const char *path,
   else
     {
       g_free (filename);
-      filename = g_build_filename (g_get_home_dir(),
+      filename = g_build_filename (g_get_home_dir (),
                                    ".thumbnails", "fail",
                                    "gnome-thumbnail-factory",
                                    basename,
@@ -1268,10 +1270,10 @@ get_thumbnail_attributes (const char *path,
 
 #ifdef G_OS_WIN32
 static void
-win32_get_file_user_info (const gchar* filename,
-			  gchar **group_name, 
-			  gchar **user_name, 
-			  gchar **real_name)
+win32_get_file_user_info (const gchar  *filename,
+			  gchar       **group_name, 
+			  gchar       **user_name, 
+			  gchar       **real_name)
 {
   PSECURITY_DESCRIPTOR psd = NULL;
   DWORD sd_size = 0; /* first call calculates the size required */
@@ -1308,8 +1310,8 @@ win32_get_file_user_info (const gchar* filename,
 			        domain, &domain_len, /* no domain info yet */
 			        &name_use)  && (ERROR_INSUFFICIENT_BUFFER != GetLastError()))
 	  break;
-	name = g_try_malloc (name_len*sizeof(wchar_t));
-	domain = g_try_malloc (domain_len*sizeof(wchar_t));
+	name = g_try_malloc (name_len * sizeof (wchar_t));
+	domain = g_try_malloc (domain_len * sizeof (wchar_t));
 	if (name && domain &&
             LookupAccountSidW (NULL, /* local machine */
                                psid, 
@@ -1335,8 +1337,8 @@ win32_get_file_user_info (const gchar* filename,
 			        domain, &domain_len, /* no domain info yet */
 			        &name_use)  && (ERROR_INSUFFICIENT_BUFFER != GetLastError()))
 	  break;
-	name = g_try_malloc (name_len*sizeof(wchar_t));
-	domain = g_try_malloc (domain_len*sizeof(wchar_t));
+	name = g_try_malloc (name_len * sizeof (wchar_t));
+	domain = g_try_malloc (domain_len * sizeof (wchar_t));
 	if (name && domain &&
             LookupAccountSidW (NULL, /* local machine */
                                psid, 
@@ -1386,11 +1388,13 @@ _g_local_file_info_get (const char             *basename,
   res = g_lstat (path, &statbuf);
   if (res == -1)
     {
+      int errsv = errno;
+
       g_object_unref (info);
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error stating file '%s': %s"),
-		   path, g_strerror (errno));
+		   path, g_strerror (errsv));
       return NULL;
     }
   
@@ -1490,7 +1494,7 @@ _g_local_file_info_get (const char             *basename,
 
 	      mimetype_icon = g_strdup (content_type);
 	      
-	      while ((p = strchr(mimetype_icon, '/')) != NULL)
+	      while ((p = strchr (mimetype_icon, '/')) != NULL)
 		*p = '-';
 
 	      p = strchr (content_type, '/');
@@ -1617,10 +1621,12 @@ _g_local_file_info_get_from_fd (int      fd,
   
   if (fstat (fd, &stat_buf) == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error stating file descriptor: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return NULL;
     }
 
@@ -1721,10 +1727,12 @@ set_unix_mode (char                       *filename,
   
   if (g_chmod (filename, val) == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting permissions: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   return TRUE;
@@ -1770,10 +1778,12 @@ set_unix_uid_gid (char                       *filename,
   
   if (res == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting owner: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
 	  return FALSE;
     }
   return TRUE;
@@ -1801,10 +1811,12 @@ set_symlink (char                       *filename,
   
   if (g_lstat (filename, &statbuf))
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting symlink: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
@@ -1818,19 +1830,23 @@ set_symlink (char                       *filename,
   
   if (g_unlink (filename))
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting symlink: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
   if (symlink (filename, val) != 0)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting symlink: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
@@ -1927,13 +1943,15 @@ set_mtime_atime (char                       *filename,
       times[1].tv_usec = val_usec;
     }
   
-  res = utimes(filename, times);
+  res = utimes (filename, times);
   if (res == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error setting owner: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
 	  return FALSE;
     }
   return TRUE;
@@ -1941,13 +1959,13 @@ set_mtime_atime (char                       *filename,
 #endif
 
 gboolean
-_g_local_file_info_set_attribute (char                       *filename,
-				  const char                 *attribute,
-				  GFileAttributeType          type,
-				  gpointer                    value_p,
-				  GFileQueryInfoFlags         flags,
-				  GCancellable               *cancellable,
-				  GError                    **error)
+_g_local_file_info_set_attribute (char                 *filename,
+				  const char           *attribute,
+				  GFileAttributeType    type,
+				  gpointer              value_p,
+				  GFileQueryInfoFlags   flags,
+				  GCancellable         *cancellable,
+				  GError              **error)
 {
   GFileAttributeValue value = { 0 };
 
