@@ -179,120 +179,198 @@ match_error (gint errcode)
   return _("unknown error");
 }
 
-static const gchar *
-compile_error (gint errcode,
-	       const gchar *default_msg)
+static void
+translate_compile_error (gint *errcode, gchar **errmsg)
 {
-  switch (errcode)
+  /* Compile errors are created adding 100 to the error code returned
+   * by PCRE.
+   * If errcode is known we put the translatable error message in
+   * erromsg. If errcode is unknown we put the generic
+   * G_REGEX_ERROR_COMPILE error code in errcode and keep the
+   * untranslated error message returned by PCRE.
+   * Note that there can be more PCRE errors with the same GRegexError
+   * and that some PCRE errors are useless for us.
+   */
+  *errcode += 100;
+
+  switch (*errcode)
     {
-    case 1:
-      return _("\\ at end of pattern");
-    case 2:
-      return _("\\c at end of pattern");
-    case 3:
-      return _("unrecognized character follows \\");
-    case 37:
-      return _("case changing escapes are not allowed here");
-    case 4:
-      return _("numbers out of order in {} quantifier");
-    case 5:
-      return _("number too big in {} quantifier");
-    case 6:
-      return _("missing terminating ] for character class");
-    case 7:
-      return _("invalid escape sequence in character class");
-    case 8:
-      return _("range out of order in character class");
-    case 9:
-      return _("nothing to repeat");
-    case 12:
-      return _("unrecognized character after (?");
-    case 24:
-      return _("unrecognized character after (?<");
-    case 41:
-      return _("unrecognized character after (?P");
-    case 13:
-      return _("POSIX named classes are supported only within a class");
-    case 14:
-      return _("(?R or (?[+-]digits must be followed by )");
-    case 15:
-      return _("reference to non-existent subpattern");
-    case 18:
-      return _("missing ) after comment");
-    case 20:
-      return _("regular expression too large");
-    case 21:
-      return _("failed to get memory");
-    case 25:
-      return _("lookbehind assertion is not fixed length");
-    case 26:
-      return _("malformed number or name after (?(");
-    case 27:
-      return _("conditional group contains more than two branches");
-    case 28:
-      return _("assertion expected after (?(");
-    case 30:
-      return _("unknown POSIX class name");
-    case 31:
-      return _("POSIX collating elements are not supported");
-    case 34:
-      return _("character value in \\x{...} sequence is too large");
-    case 35:
-      return _("invalid condition (?(0)");
-    case 36:
-      return _("\\C not allowed in lookbehind assertion");
-    case 40:
-      return _("recursive call could loop indefinitely");
-    case 42:
-      return _("missing terminator in subpattern name");
-    case 43:
-      return _("two named subpatterns have the same name");
-    case 46:
-      return _("malformed \\P or \\p sequence");
-    case 47:
-      return _("unknown property name after \\P or \\p");
-    case 48:
-      return _("subpattern name is too long (maximum 32 characters)");
-    case 49:
-      return _("too many named subpatterns (maximum 10,000)");
-    case 51:
-      return _("octal value is greater than \\377");
-    case 54:
-      return _("DEFINE group contains more than one branch");
-    case 55:
-      return _("repeating a DEFINE group is not allowed");
-    case 56:
-      return _("inconsistent NEWLINE options");
-    case 57:
-      return _("\\g is not followed by a braced name or an optionally "
-	       "braced non-zero number");
+    case G_REGEX_ERROR_STRAY_BACKSLASH:
+      *errmsg = _("\\ at end of pattern");
+      break;
+    case G_REGEX_ERROR_MISSING_CONTROL_CHAR:
+      *errmsg = _("\\c at end of pattern");
+      break;
+    case G_REGEX_ERROR_UNRECOGNIZED_ESCAPE:
+      *errmsg = _("unrecognized character follows \\");
+      break;
+    case 137:
+      /* A number of Perl escapes are not handled by PCRE.
+       * Therefore it explicitly raises ERR37.
+       */
+      *errcode = G_REGEX_ERROR_UNRECOGNIZED_ESCAPE;
+      *errmsg = _("case-changing escapes (\\l, \\L, \\u, \\U) are not allowed here");
+      break;
+    case G_REGEX_ERROR_QUANTIFIERS_OUT_OF_ORDER:
+      *errmsg = _("numbers out of order in {} quantifier");
+      break;
+    case G_REGEX_ERROR_QUANTIFIER_TOO_BIG:
+      *errmsg = _("number too big in {} quantifier");
+      break;
+    case G_REGEX_ERROR_UNTERMINATED_CHARACTER_CLASS:
+      *errmsg = _("missing terminating ] for character class");
+      break;
+    case G_REGEX_ERROR_INVALID_ESCAPE_IN_CHARACTER_CLASS:
+      *errmsg = _("invalid escape sequence in character class");
+      break;
+    case G_REGEX_ERROR_RANGE_OUT_OF_ORDER:
+      *errmsg = _("range out of order in character class");
+      break;
+    case G_REGEX_ERROR_NOTHING_TO_REPEAT:
+      *errmsg = _("nothing to repeat");
+      break;
+    case G_REGEX_ERROR_UNRECOGNIZED_CHARACTER:
+      *errmsg = _("unrecognized character after (?");
+      break;
+    case 124:
+      *errcode = G_REGEX_ERROR_UNRECOGNIZED_CHARACTER;
+      *errmsg = _("unrecognized character after (?<");
+      break;
+    case 141:
+      *errcode = G_REGEX_ERROR_UNRECOGNIZED_CHARACTER;
+      *errmsg = _("unrecognized character after (?P");
+      break;
+    case G_REGEX_ERROR_POSIX_NAMED_CLASS_OUTSIDE_CLASS:
+      *errmsg = _("POSIX named classes are supported only within a class");
+      break;
+    case G_REGEX_ERROR_UNMATCHED_PARENTHESIS:
+      *errmsg = _("missing terminating )");
+      break;
+    case 122:
+      *errcode = G_REGEX_ERROR_UNMATCHED_PARENTHESIS;
+      *errmsg = _(") without opening (");
+      break;
+    case 129:
+      *errcode = G_REGEX_ERROR_UNMATCHED_PARENTHESIS;
+      /* translators: '(?R' and '(?[+-]digits' are both meant as (groups of) 
+       * sequences here, '(?-54' would be an example for the second group.
+       */
+      *errmsg = _("(?R or (?[+-]digits must be followed by )");
+      break;
+    case G_REGEX_ERROR_INEXISTENT_SUBPATTERN_REFERENCE:
+      *errmsg = _("reference to non-existent subpattern");
+      break;
+    case G_REGEX_ERROR_UNTERMINATED_COMMENT:
+      *errmsg = _("missing ) after comment");
+      break;
+    case G_REGEX_ERROR_EXPRESSION_TOO_LARGE:
+      *errmsg = _("regular expression too large");
+      break;
+    case G_REGEX_ERROR_MEMORY_ERROR:
+      *errmsg = _("failed to get memory");
+      break;
+    case G_REGEX_ERROR_VARIABLE_LENGTH_LOOKBEHIND:
+      *errmsg = _("lookbehind assertion is not fixed length");
+      break;
+    case G_REGEX_ERROR_MALFORMED_CONDITION:
+      *errmsg = _("malformed number or name after (?(");
+      break;
+    case G_REGEX_ERROR_TOO_MANY_CONDITIONAL_BRANCHES:
+      *errmsg = _("conditional group contains more than two branches");
+      break;
+    case G_REGEX_ERROR_ASSERTION_EXPECTED:
+      *errmsg = _("assertion expected after (?(");
+      break;
+    case G_REGEX_ERROR_UNKNOWN_POSIX_CLASS_NAME:
+      *errmsg = _("unknown POSIX class name");
+      break;
+    case G_REGEX_ERROR_POSIX_COLLATING_ELEMENTS_NOT_SUPPORTED:
+      *errmsg = _("POSIX collating elements are not supported");
+      break;
+    case G_REGEX_ERROR_HEX_CODE_TOO_LARGE:
+      *errmsg = _("character value in \\x{...} sequence is too large");
+      break;
+    case G_REGEX_ERROR_INVALID_CONDITION:
+      *errmsg = _("invalid condition (?(0)");
+      break;
+    case G_REGEX_ERROR_SINGLE_BYTE_MATCH_IN_LOOKBEHIND:
+      *errmsg = _("\\C not allowed in lookbehind assertion");
+      break;
+    case G_REGEX_ERROR_INFINITE_LOOP:
+      *errmsg = _("recursive call could loop indefinitely");
+      break;
+    case G_REGEX_ERROR_MISSING_SUBPATTERN_NAME_TERMINATOR:
+      *errmsg = _("missing terminator in subpattern name");
+      break;
+    case G_REGEX_ERROR_DUPLICATE_SUBPATTERN_NAME:
+      *errmsg = _("two named subpatterns have the same name");
+      break;
+    case G_REGEX_ERROR_MALFORMED_PROPERTY:
+      *errmsg = _("malformed \\P or \\p sequence");
+      break;
+    case G_REGEX_ERROR_UNKNOWN_PROPERTY:
+      *errmsg = _("unknown property name after \\P or \\p");
+      break;
+    case G_REGEX_ERROR_SUBPATTERN_NAME_TOO_LONG:
+      *errmsg = _("subpattern name is too long (maximum 32 characters)");
+      break;
+    case G_REGEX_ERROR_TOO_MANY_SUBPATTERNS:
+      *errmsg = _("too many named subpatterns (maximum 10,000)");
+      break;
+    case G_REGEX_ERROR_INVALID_OCTAL_VALUE:
+      *errmsg = _("octal value is greater than \\377");
+      break;
+    case G_REGEX_ERROR_TOO_MANY_BRANCHES_IN_DEFINE:
+      *errmsg = _("DEFINE group contains more than one branch");
+      break;
+    case G_REGEX_ERROR_DEFINE_REPETION:
+      *errmsg = _("repeating a DEFINE group is not allowed");
+      break;
+    case G_REGEX_ERROR_INCONSISTENT_NEWLINE_OPTIONS:
+      *errmsg = _("inconsistent NEWLINE options");
+      break;
+    case G_REGEX_ERROR_MISSING_BACK_REFERENCE:
+      *errmsg = _("\\g is not followed by a braced name or an optionally "
+		 "braced non-zero number");
+      break;
     case 11:
-      return _("unexpected repeat");
+      *errcode = G_REGEX_ERROR_INTERNAL;
+      *errmsg = _("unexpected repeat");
+      break;
     case 23:
-      return _("code overflow");
+      *errcode = G_REGEX_ERROR_INTERNAL;
+      *errmsg = _("code overflow");
+      break;
     case 52:
-      return _("overran compiling workspace");
+      *errcode = G_REGEX_ERROR_INTERNAL;
+      *errmsg = _("overran compiling workspace");
+      break;
     case 53:
-      return _("previously-checked referenced subpattern not found");
+      *errcode = G_REGEX_ERROR_INTERNAL;
+      *errmsg = _("previously-checked referenced subpattern not found");
+      break;
     case 16:
       /* This should not happen as we never pass a NULL erroffset */
       g_warning ("erroffset passed as NULL");
+      *errcode = G_REGEX_ERROR_COMPILE;
       break;
     case 17:
       /* This should not happen as we check options before passing them
        * to pcre_compile2() */
       g_warning ("unknown option bit(s) set");
+      *errcode = G_REGEX_ERROR_COMPILE;
       break;
     case 32:
     case 44:
     case 45:
       /* These errors should not happen as we are using an UTF8-enabled PCRE
        * and we do not check if strings are valid */
-      g_warning (default_msg);
+      g_warning (*errmsg);
+      *errcode = G_REGEX_ERROR_COMPILE;
       break;
+    default:
+      *errcode = G_REGEX_ERROR_COMPILE;
     }
-
-  return default_msg;
 }
 
 /* GMatchInfo */
@@ -956,9 +1034,6 @@ g_regex_unref (GRegex *regex)
  * Compiles the regular expression to an internal form, and does 
  * the initial setup of the #GRegex structure.  
  * 
- * The error code of @error is G_REGEX_ERROR_COMPILE, but
- * future versions will add more specific error codes.
- * 
  * Returns: a #GRegex structure. Call g_regex_unref() when you 
  *   are done with it
  *
@@ -972,7 +1047,7 @@ g_regex_new (const gchar         *pattern,
 {
   GRegex *regex;
   pcre *re;
-  const gchar *errmsg;
+  gchar *errmsg;
   gint erroffset;
   gint errcode;
   gboolean optimize = FALSE;
@@ -1038,8 +1113,8 @@ g_regex_new (const gchar         *pattern,
     }
 
   /* compile the pattern */
-  re = pcre_compile2 (pattern, compile_options, &errcode, &errmsg,
-		      &erroffset, NULL);
+  re = pcre_compile2 (pattern, compile_options, &errcode,
+		      (const gchar **)&errmsg, &erroffset, NULL);
 
   /* if the compilation failed, set the error member and return 
    * immediately */
@@ -1047,15 +1122,14 @@ g_regex_new (const gchar         *pattern,
     {
       GError *tmp_error;
 
-      /* Get the translatable error message instead of just using the
-       * English one */
-      errmsg = compile_error (errcode, errmsg);
+      /* Translate the PCRE error code to GRegexError and use a translated
+       * error message if possible */
+      translate_compile_error (&errcode, &errmsg);
 
       /* PCRE uses byte offsets but we want to show character offsets */
       erroffset = g_utf8_pointer_to_offset (pattern, &pattern[erroffset]);
 
-      tmp_error = g_error_new (G_REGEX_ERROR,
-			       G_REGEX_ERROR_COMPILE,
+      tmp_error = g_error_new (G_REGEX_ERROR, errcode,
 			       _("Error while compiling regular "
 				 "expression %s at char %d: %s"),
 			       pattern, erroffset, errmsg);
@@ -1087,7 +1161,7 @@ g_regex_new (const gchar         *pattern,
 
   if (optimize)
     {
-      regex->extra = pcre_study (regex->pcre_re, 0, &errmsg);
+      regex->extra = pcre_study (regex->pcre_re, 0, (const gchar **)&errmsg);
       if (errmsg != NULL)
         {
           GError *tmp_error = g_error_new (G_REGEX_ERROR,
@@ -1227,7 +1301,7 @@ g_regex_match_simple (const gchar        *pattern,
  * To retrieve all the non-overlapping matches of the pattern in 
  * string you can use g_match_info_next().
  *
- * <informalexample><programlisting>
+ * |[
  * static void
  * print_uppercase_words (const gchar *string)
  * {
@@ -1247,7 +1321,7 @@ g_regex_match_simple (const gchar        *pattern,
  *   g_match_info_free (match_info);
  *   g_regex_unref (regex);
  * }
- * </programlisting></informalexample>
+ * ]|
  *
  * Returns: %TRUE is the string matched, %FALSE otherwise
  *
@@ -1296,7 +1370,7 @@ g_regex_match (const GRegex      *regex,
  * To retrieve all the non-overlapping matches of the pattern in 
  * string you can use g_match_info_next().
  *
- * <informalexample><programlisting>
+ * |[
  * static void
  * print_uppercase_words (const gchar *string)
  * {
@@ -1322,7 +1396,7 @@ g_regex_match (const GRegex      *regex,
  *       g_error_free (error);
  *     }
  * }
- * </programlisting></informalexample>
+ * ]|
  *
  * Returns: %TRUE is the string matched, %FALSE otherwise
  *
